package v1

import (
	"encoding/json"
	"lift-monitor/model"
	"strings"
	"time"
)

type LiftLoopData struct {
	Sn                  string  `json:"sn"`
	ProjectId           int64   `json:"project_id"`
	LiftNo              int64   `json:"lift_no"`
	LiftDirect          int     `json:"lift_direct"` //0 左吊笼 1 右吊笼
	StartTime           int64   `json:"start_time"`
	EndTime             int64   `json:"end_time"`
	DriverName          string  `json:"driver_name"`
	DriverCertID        string  `json:"driver_cert_id"`
	Weight              int64   `json:"weight"`
	WeightPercent       int64   `json:"weight_percent"`
	HighStart           float64 `json:"high_start"`
	HighEnd             float64 `json:"high_end"`
	HighTrip            float64 `json:"high_trip"`
	Direction           int     `json:"direction"`
	Speed               float64 `json:"speed"`
	MaxTiltX            float64 `json:"max_tilt_x"`
	MaxTiltY            float64 `json:"max_tilt_y"`
	People              int     `json:"people"`
	WeightAlarmState    int     `json:"weight_alarm_state"`
	AuthAlarmState      int     `json:"auth_alarm_state"`
	SpeedAlarmState     int     `json:"speed_alarm_state"`
	HighAlarmState      int     `json:"high_alarm_state"`
	PeopleAlarmState    int     `json:"people_alarm_state"`
	TiltAlarmState      int     `json:"tilt_alarm_state"`
	WindSpeedAlarmState int     `json:"wind_speed_alarm_state"`
	MaxWindSpeed        float64 `json:"max_wind_speed"`
}

func LiftLoopDataWrite(req *LiftLoopData) error {
	tags := map[string]string{"sn": req.Sn}
	fields := map[string]interface{}{}

	bytes, _ := json.Marshal(req)
	json.Unmarshal(bytes, &fields)
	delete(fields, "sn")
	t := time.Now()

	// 写数据
	err := model.WriteLiftData(req.Sn, tags, fields, t, "loop_data")
	if err != nil {
		return err
	}
	return nil
}

func LiftFrameLoopDataRequestHandle(sn string, liftNo byte, version byte, data []byte) (res []byte, err error) {
	projectId, _ := checkAuth(sn)
	if projectId <= 0 {
		// 未认证或需重新认证
		return makeResponse(sn, liftNo, LiftFrameLoopDataResponse, version, []byte{0xfe}), nil
	}
	if len(data) < 32 {
		return makeResponse(sn, liftNo, LiftFrameLoopDataResponse, version, []byte{0xff}), nil
	}
	record := &LiftLoopData{}
	record.Sn = sn
	record.LiftNo = int64(liftNo) >> 1
	record.LiftDirect = int(liftNo) & 1
	record.ProjectId = projectId

	// 时间戳
	record.StartTime = int64(data[0])<<24 + int64(data[1])<<16 + int64(data[2])<<8 + int64(data[3])

	record.EndTime = int64(data[4])<<24 + int64(data[5])<<16 + int64(data[6])<<8 + int64(data[7])

	// 驾驶员姓名
	str := string(data[8:40])
	str = strings.TrimSpace(str)
	record.DriverName = str

	// 驾驶员身份证号
	str = string(data[40:58])
	str = strings.TrimSpace(str)
	record.DriverCertID = str
	// 本次载重
	record.Weight = int64(data[58]<<8) + int64(data[59])
	// 本次载重百分比
	record.WeightPercent = int64(data[60])
	// 本次起升起点高度
	record.HighStart = (float64(int16(data[61])<<8) + float64(data[62])) * 0.1
	// 本次起升终点高度
	record.HighEnd = (float64(int16(data[63])<<8) + float64(data[64])) * 0.1
	// 本次起升行程
	record.HighTrip = (float64(int16(data[65])<<8) + float64(data[66])) * 0.1
	// 本次起升方向
	record.Direction = int(data[67])
	// 本次起升平均速度
	record.Speed = (float64(int16(data[68])<<8) + float64(data[69])) * 0.1
	// 本次起升最大X向倾斜度
	record.MaxTiltX = (float64(int16(data[70])<<8) + float64(data[71])) * 0.1
	// 本次起升最大Y向倾斜度
	record.MaxTiltY = (float64(int16(data[72])<<8) + float64(data[73])) * 0.1
	// 本次装载人数
	record.People = int(data[74])
	// 重量状态 00重量正常 01重量预警 10重量报警
	record.WeightAlarmState = int(data[75]) & 0x3
	// 身份认证状态 00 身份认证正常 01身份认证异常
	record.AuthAlarmState = (int(data[75]) >> 2) & 0x3
	// 速度状态 00重量正常 01重量预警 10重量报警
	record.SpeedAlarmState = (int(data[75]) >> 4) & 0x3
	// 高度状态 00重量正常 01重量预警 10重量报警
	record.HighAlarmState = (int(data[75]) >> 6) & 0x3

	// 人数状态 00人数正常 01人数超载
	record.PeopleAlarmState = int(data[76]) & 0x3
	// 倾角状态 00重量正常 01重量预警 10重量报警
	record.TiltAlarmState = (int(data[76]) >> 2) & 0x3
	// 风速状态 00重量正常 01重量预警 10重量报警
	record.WindSpeedAlarmState = (int(data[76]) >> 4) & 0x3

	// 本次起升最大风速
	record.MaxWindSpeed = (float64(int16(data[77])<<8) + float64(data[78])) * 0.1

	if LiftLoopDataWrite(record) != nil {
		return makeResponse(sn, liftNo, LiftFrameLoopDataResponse, version, []byte{0xfd}), nil
	}

	return makeResponse(sn, liftNo, LiftFrameLoopDataResponse, version, []byte{0x00}), nil
}
