package v1

import (
	"encoding/json"
	"fmt"
	"lift-monitor/model"
	"strconv"
	"strings"
	"time"
)

type LiftRealtimeData struct {
	DateTime            int64   `json:"date_time"`
	Weight              int64   `json:"weight"`
	WeightPercent       int64   `json:"weight_percent"`
	People              int64   `json:"people"`
	High                float64 `json:"high"`
	HighPercent         int64   `json:"high_percent"`
	Speed               float64 `json:"speed"`
	Direction           int     `json:"direction"`
	Tilt1               float64 `json:"tilt1"`
	Tilt1Percent        int64   `json:"tilt1_percent"`
	Tilt2               float64 `json:"tilt2"`
	Tilt2Percent        int64   `json:"tilt2_percent"`
	DriverNo            int64   `json:"driver_no"`
	FrontDoorState      int     `json:"front_door_state"`
	BackDoorState       int     `json:"back_door_state"`
	WeightAlarmState    int     `json:"weight_alarm_state"`
	HighAlarmState      int     `json:"high_alarm_state"`
	SpeedAlarmState     int     `json:"speed_alarm_state"`
	PeopleAlarmState    int     `json:"people_alarm_state"`
	TiltAlarmState      int     `json:"tilt_alarm_state"`
	FrontDoorAlarmState int     `json:"front_door_alarm_state"`
	BackDoorAlarmState  int     `json:"back_door_alarm_state"`
	WindSpeedAlarmState int     `json:"wind_speed_alarm_state"`
	WindSpeed           float64 `json:"wind_speed"`
	Lng        float64 `json:"lng"`
	Lat        float64 `json:"lat"`
	Sn         string  `json:"sn"`
	ProjectId  int64   `json:"project_id"`
	LiftNo     int64   `json:"lift_no"`
	LiftDirect int     `json:"lift_direct"` //0 左吊笼 1 右吊笼
	IsAlarm int `json:"is_alarm"`
}

func LiftRealtimeDataWrite(req *LiftRealtimeData) error {
	tags := map[string]string{"sn": req.Sn}
	fields := map[string]interface{}{}

	bytes, _ := json.Marshal(req)
	json.Unmarshal(bytes, &fields)
	delete(fields, "sn")
	t := time.Now()

	// 写数据
	err := model.WriteLiftData(req.Sn, tags, fields, t, "realtime_data")
	if err != nil {
		return err
	}
	return nil
}

func LiftFrameRealtimeDataRequestHandle(sn string, liftNo byte, version byte, data []byte) (res []byte, err error) {
	hexPrintf(data)
	projectId, devicename := checkAuth(sn)
	if projectId <= 0 {
		// 未认证或需重新认证
		return makeResponse(sn, liftNo, LiftFrameRealtimeDataResponse, version, []byte{0xfe}), nil
	}
	if len(data) < 46 {
		return makeResponse(sn, liftNo, LiftFrameRealtimeDataResponse, version, []byte{0xff}), nil
	}
	record := &LiftRealtimeData{}
	record.Sn = sn
	record.LiftNo = int64(liftNo) >> 1
	record.LiftDirect = int(liftNo) & 1
	record.ProjectId = projectId
	// 时间戳
	record.DateTime = int64(data[0])<<24 + int64(data[1])<<16 + int64(data[2])<<8 + int64(data[3])

	// 实时起重量
	record.Weight = int64(data[4]) << 8 + int64(data[5])
	// 起重量百分比
	record.WeightPercent = int64(data[6])
	// 实时人数
	record.People = int64(data[7])
	// 实时高度
	record.High = (float64(int16(data[8])<<8) + float64(data[9])) * 0.1
	// 高度百分比
	record.HighPercent = int64(data[10])
	// 实时速度
	record.Speed = (float64(int16(data[11])<<8) + float64(data[12])) * 0.1
	// 方向
	record.Direction = int(data[13])
	// 实时倾角1
	record.Tilt1 = (float64(int16(data[14])<<8) + float64(data[15])) * 0.1
	// 倾角百分比1
	record.Tilt1Percent = int64(data[16])
	// 实时倾角2
	record.Tilt2 = (float64(int16(data[17])<<8) + float64(data[18])) * 0.1
	// 倾角百分比2
	record.Tilt2Percent = int64(data[19])
	// 驾驶员工号 0 未认证
	record.DriverNo = int64(data[20])
	// 前门状态 0 关 1 开
	record.FrontDoorState = int(data[21])&0x1
	// 后门状态
	record.BackDoorState = (int(data[21]) >> 1) &0x1
	// 重量异常信息 0 正常 1 预警 2 报警
	record.WeightAlarmState = int(data[22]) & 0x3
	// 高度异常信息 0 正常 1 预警 2 报警
	record.HighAlarmState = (int(data[22]) >> 2) & 0x3
	// 速度异常信息 0 正常 1 预警 2 报警
	record.WindSpeedAlarmState = (int(data[22]) >> 4) & 0x3
	// 人数异常信息 0 正常 1 预警 2 报警
	record.PeopleAlarmState = (int(data[22]) >> 4) & 0x3


	// 倾角异常信息 0 正常 1 预警 2 报警
	record.TiltAlarmState = int(data[23]) & 0x3
	// 前门锁异常信息 0 正常 1 异常
	record.FrontDoorAlarmState = int(data[23]) >> 2
	// 后门锁异常信息 0 正常 1 异常
	record.BackDoorAlarmState = int(data[23]) >> 3
	// 风速异常信息 0 正常 1 预警 2 报警
	record.WindSpeedAlarmState = (int(data[23]) >> 4) & 0x3

	// 实时风速
	record.WindSpeed = (float64(int16(data[24])<<8) + float64(data[25])) * 0.1
	// 经度
	str := string(data[26:38])
	str = strings.TrimSpace(str)
	record.Lng, _ = strconv.ParseFloat(str, 64)

	fmt.Printf("lng:%s,0x%02x\n", str, data[26])
	// 纬度
	str = string(data[38:50])
	fmt.Printf("lat:%s,0x%02x\n", str, data[38])
	str = strings.TrimSpace(str)
	record.Lat, _ = strconv.ParseFloat(str, 64)

	isAlarm := handleAlarms(record, sn, int64(liftNo), record.DateTime, projectId, devicename)

	if isAlarm {
		record.IsAlarm = 1
	}
	if LiftRealtimeDataWrite(record) != nil {
		return makeResponse(sn, liftNo, LiftFrameRealtimeDataResponse, version, []byte{0xfd}), nil
	}


	return makeResponse(sn, liftNo, LiftFrameRealtimeDataResponse, version, []byte{0x00}), nil
}
